/*******************************************************************************
 * Copyright (C) 1999-2011 Maxim Integrated Products, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL MAXIM INTEGRATED PRODUCTS BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Maxim Integrated Products
 * shall not be used except as stated in the Maxim Integrated Products
 * Branding Policy.
 *
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Maxim Integrated Products retains all ownership rights.
 *
 *  Module Name: DS4830 flash read and write 'C' routine
 *  Description: Demonstrates flash read write 'C' routines interface with 
 *               assembly flash read and write functions
 *  Filename:    flash.c
 *  Author:      SKJ
 *  Compiler:    IAR Compiler version 2.20I
 *
 *******************************************************************************
 */

#include <intrinsics.h>
#include <iods4830.h>
#include <stdio.h>

#include "flash.h"
#include "memory.h"

unsigned char ucDisableFlash = 0;

/*******************************************************************************
 * Function: RecallConfigFromFlash
 *
 * This function reads data from flash and write into SRAM.
 *
 * Input:     None
 * Output:    None
 *
 * Example:   To Recall (read) block of data from flash into sram...
 *                  RecallConfigFromFlash();
 *********
 * Notes:
 *********
 * 1.
 * During block read from flash global interrupt is disabled.
 *
 * 2.
 * Last parameter into read_block_from_flash() function is "No of WORDS" to be
 * read from flash.
 *
 ******************************************************************************/
void RecallConfigFromFlash(void)
{
      
     //Disable Global Interrupt
     __disable_interrupt();

     //Read data from flash and store into sram
     //Assembly function call
     read_block_from_flash(CONFIG_FLASH, CONFIG_SRAM, (FLASH_DATA_SIZE0 + FLASH_DATA_SIZE1 + 4)>>1);

     //Enable Global Interrupt
     __enable_interrupt();
}


/*******************************************************************************
 * Function: StoreConfigInFlash
 *
 * This function reads data from flash and write into SRAM.
 *
 * Input:     None
 * Output:    int
 *            0 if success
 *            non-zero if fail
 *
 * Example:   To Store block of data from sram into flash.
 *                  int StoreConfigInFlash();
 *********
 * Notes:
 *********
 * 1.
 * During block read from flash, global interrupt is disabled.
 *
 * 2.
 * Last parmeter into write_block_to_flash() function is "No of WORDS" to be
 * written into flash. This should be less than page size (256 Words)
 *
 * 3.
 * In write_block_to_flash assembly function, data is written in word mode.
 * See Device Datasheet for word write time.
 *
 ******************************************************************************/
int StoreConfigInFlash(void)
{
     int FlashFail = -1;
      
     //Disable Global Interrupt
     __disable_interrupt();

     //Flash Page Erase
     //Assembly Function Calls
      FlashFail = clear_block_in_flash(CONFIG_FLASH);

      //Write Block of data from SRAM to Flash
      //Assembly call
      if(!FlashFail)  //Write new Configuration in flash is no flash failure
           FlashFail = write_block_to_flash(CONFIG_FLASH, CONFIG_SRAM, (FLASH_DATA_SIZE0 + FLASH_DATA_SIZE1 + 4)>>1);
 
     //Enable Global Interrupt
     __enable_interrupt();
     
     return FlashFail;
}

//End
